function [TP FP TN FN] = eval_separation(y, label, max_points)

% [TP FP TN FN] = eval_separation(y, label, max_points)
%
% Calculates the number of true positives (TP), false positives (FP),
% true negatives (TN) and false negatives (FN) given a continuous
% prediction score and a label.
%
% y -- a continuous prediction score
% label -- a corresponding label (only position wher label is either -1
%   or +1 are considered)
% max_points -- the maximum number of points where TP, FP, TN and FN are
%   assessed 
% returns true positives (TP), false positives (FP), true negatives (TN)
%   and false negatives (FN) each is an k x m matrix where k is equal to
%   the number of rows in y and m is the number of reasonable thresholds
%   (at most max_points). 
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008

if ~exist('max_points', 'var'),
  max_points = 200;
end

y_min = min(y(:));
y_max = max(y(:));

if size(y,2)>max_points,
  % interpolated curves
  thresholds = [y_min-1 linspace(y_min, y_max, max_points) y_max+1];
  num_points = length(thresholds);
  TP = zeros(size(y,1),num_points);
  FP = zeros(size(y,1),num_points);
  TN = zeros(size(y,1),num_points);
  FN = zeros(size(y,1),num_points);

  for i=1:size(y,1),
    assert(~any(isnan(y(i,:))));
    y_tmp = y(i,:);
    
    for j=1:num_points,
      is_positive = y_tmp>=thresholds(j);
      TP(i,j) = sum(label == +1 & is_positive);
      FP(i,j) = sum(label == -1 & is_positive);
      TN(i,j) = sum(label == -1 & ~is_positive);
      FN(i,j) = sum(label == +1 & ~is_positive);
    end
  end
else
  % exact curves
  for i=1:size(y,1),
    assert(~any(isnan(y(i,:))));
    y_tmp = y(i,:);
    
    thresholds = [y_min-1 sort(y_tmp) y_max+1];
    num_points = length(thresholds);
    for j=1:num_points,
      is_positive = y_tmp>=thresholds(j);
      TP(i,j) = sum(label == +1 & is_positive);
      FP(i,j) = sum(label == -1 & is_positive);
      TN(i,j) = sum(label == -1 & ~is_positive);
      FN(i,j) = sum(label == +1 & ~is_positive);
    end
    assert(all(TP(i,:)+FN(i,:) == TP(i,1)+FN(i,1)));
    assert(all(TN(i,:)+FP(i,:) == TN(i,1)+FP(i,1)));
  end
end
