function arrays = norm_quantiles(arrays)

% arrays = norm_quantiles(arrays)
%
% Quantile normalization for tiling array intensities as proposed in
% Bolstad et al., Bioinformatics 2003, A comparison of normalization
% methods for high density oligonuleotide array data based on variance
% and bias.
%
% arrays -- a cell of array structs (see read_array_CEL.m)
% returns normalized arrays
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-2008
  
NUM_ARRAYS = length(arrays);
VERBOSE = 0;

%%% sort and concatenate sorted intensity data (columns) 

fprintf('sorting intensity data...\n');
I_sorted   = [];
permut_idx = [];
I_mean     = [];
nan_idx    = cell(1,NUM_ARRAYS);
for a=1:NUM_ARRAYS,
  if a==1,
    I_sorted   = zeros(arrays(a).num_cells,NUM_ARRAYS); 
    permut_idx = zeros(arrays(a).num_cells,NUM_ARRAYS); 
  end
  I_tmp = reshape(arrays(a).mean,1,arrays(a).num_cells);
  
  % deal with nan values
  nan_idx{a} = find(isnan(I_tmp));
  num_nan = length(nan_idx{a});
  fprintf('  substituting %i NaN values (%2.1f%%) by sampled intensities\n', ...
          num_nan, 100*num_nan/length(I_tmp));
  if num_nan>0,
    pseudo_sample = setdiff(randperm(size(I_sorted,1)),nan_idx{a});
    pseudo_sample = I_tmp(pseudo_sample(1:num_nan));
    I_tmp(nan_idx{a}) = pseudo_sample;
  end
  
  [I_sorted(:,a), permut_idx(:,a)] = sort(I_tmp');
end

% plot intensity histograms
if VERBOSE>=1,
  I_tmp = log2(I_sorted);
  I_min = min(min(I_tmp));
  I_max = max(max(I_tmp));
  
  bins = [0 I_min+0.2:0.2:I_max inf];
  hists = zeros(size(I_tmp,2), length(bins));
  for i=1:size(I_sorted,2),
    hists(i,:) = histc(I_tmp(:,i),bins);
  end
  clear I_tmp
  
  colors = 'rgbycm'
  figure
  hold on
  for i=1:size(I_sorted,2),
    plot(hists(i,:), colors(i));
  end
end

%%% compute row means 

fprintf('computing row means...\n');
I_mean = mean(I_sorted,2);


% plot intensity histograms
if VERBOSE>=1
  I_tmp = log2(I_mean);
  mean_hist = histc(I_tmp,bins);
  plot(mean_hist, 'k');
  xlabel('Log intensity');
  ylabel('number of probes');
  print('-depsc', '-r300', 'I_hist_chip_QN.eps');
end

%%% map mean intensities back to specific probes on specific arrays

fprintf('computing backmapping...\n');
for a=1:NUM_ARRAYS,
  % compute inverse of permut_idx
  [I_tmp inv_idx] = sort(permut_idx(:,a));
  
  arrays(a).data_type = 'QUANTILE_NORMALIZED';
  arrays(a).raw_mean  = arrays(a).mean; 
  arrays(a).mean = round(reshape(I_mean(inv_idx), ...
                                 arrays(a).num_rows, ...
                                 arrays(a).num_cols));
  if ~isempty(nan_idx{a}),
    arrays(a).mean(nan_idx{a}) = nan;
  end
end
fprintf('finished normalization\n\n');

if VERBOSE>=2,
  figure
  hold on
  plot(arrays(a).raw_mean(10000:20000), '.k');
  plot(arrays(a).mean(10000:20000), '.r');
end