% Evaluates sequence normalization methods
%
% written by Georg Zeller, MPI Tuebingen, Germany, 2007-208

set_paths

% switches to evaluate sequence quantile normalization (SQN), support vector
% regression (SVR) or Ridge regression (RR)
OPT.TEST_SQN        = 1;
OPT.TEST_SVR        = 1;
OPT.TEST_RR         = 1;

% length of oligonucleotide probes
OPT.PROBE_LEN       = 25;
% probe labelling according to annotation
OPT.LABEL           = get_label_set();

% number of discrete gene expression levels
OPT.NUM_QUANTILES = 10;

% C hyperparameter of the SVM
OPT.SVR.C           = 10;
% maximal substring length
OPT.SVR.wdk_order   = 3;
% tolerance interval that is loss-free in training
OPT.SVR.tube        = 0.1;

% regularization strength
OPT.RR.lambda       = 0.001;
% maximal substring length
OPT.RR.order        = 3;

% number of subsets for crossvalidation
OPT.NUM_SUBSETS     = 5;
% option to restrict the subsets for crossvalidation
OPT.SUBSETS_TO_USE  = [];
% number of quantiles each of which is used by a separate predictor
OPT.NUM_PREDICTORS  = 20;

% option to also consider transcript distance features to correct for
% intensity bias caused by oligo-dT amplificaton
OPT.DIST_CORRECTION = 0;
% switch to evaluate on test subset (on validation set if 0)
OPT.USE_TEST_DATA   = 1;
% specifies the version of the gene annotation
OPT.ANNOTATION_TAG  = 'TAIR7';
% switch to save evaluation information with predictors
OPT.SAVE_PREDICTORS = 1;

% directories where predictors are stored
OPT.common_out_dir = '../../PSB_data/norm_probe_results/';
OPT.SVR_DIR = sprintf('%ssvr_result_2008-03-04_15h57_fold', OPT.common_out_dir);
OPT.RR_DIR  = sprintf('%srr_result_2008-03-04_13h09_fold', OPT.common_out_dir);
OPT.SQN_DIR = sprintf('%ssqn_result_2008-03-04_13h13', OPT.common_out_dir);

% options to load test/validation prediction if they have been computed
% and stored previously
OPT.LOAD_SVR_PRED = 1;
OPT.LOAD_RR_PRED  = 1;

% directory where array data is stored
OPT.data_dir = '../../PSB_data/array_data/';
% file names of arrays to train on 
% (if more than 1 is given, intensities are averaged)
OPT.fn_arrays = { ...
%    'T_003_tA_R_A_QN', ...
    'T_003_tB_R_A_QN', ...
%    'T_003_tC_R_A_QN' ...
                };

OPT.WRITE_NORMALIZED_I = 0;

OPT.VERBOSE = 0;

OPT


%%%%%%%%%%%%%%%%%%%%%
%%%   load data   %%%
%%%%%%%%%%%%%%%%%%%%%

for f=1:length(OPT.fn_arrays),
  if f==1,
    fprintf('loading data for array %s...\n', OPT.fn_arrays{f});
    tmp = load([OPT.data_dir OPT.fn_arrays{f} '_' OPT.ANNOTATION_TAG]);
    probe_id         = tmp.probe_id;
    probe_seq        = tmp.probe_seq;
    probe_PM_I       = tmp.probe_PM_I;
    probe_label      = tmp.probe_label;
    genes            = tmp.genes;
    probe_props      = tmp.probe_props;
  else
    fprintf('loading data for array %s...\n', OPT.fn_arrays{f});
    tmp = load([OPT.data_dir OPT.fn_arrays{f} '_' OPT.ANNOTATION_TAG], ...
               'probe_PM_I');
    probe_PM_I = [probe_PM_I; tmp.probe_PM_I];
  end
end
clear tmp f


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   summarize intensities across replicate arrays   %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% log_2 transform intensities
PM_I = log2(probe_PM_I);
if length(OPT.fn_arrays)>1,
fprintf('summarizing intensities across replicates...\n');
  PM_I = summ_replicates(PM_I);
end
clear probe_PM_I


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%   test and comparison of normalizations   %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if OPT.DIST_CORRECTION,
  [genes, PM_I_transcript, dist] ...
      = transcript_intensities_3p_bias(PM_I, probe_id, probe_label, ...
                                       genes, OPT.LABEL);
else
  [genes, PM_I_transcript] ...
      = median_transcript_intensities(PM_I, probe_id, probe_label, ...
                                    genes, OPT.LABEL);
  dist = zeros(size(PM_I));
end

if OPT.TEST_SQN,
  % load sequence-based quantile normalized intensities
  tmp = load([OPT.SQN_DIR '/I_n_iter100.mat'], 'I_n');
  PM_I_n_SQN = tmp.I_n;
  fprintf('\nEvaluating Sequence-quantile normalization\n');
  fprintf('Evaluating on %2.2f%% of probes:\n', ...
          100*mean(~isnan(PM_I_n_SQN) & PM_I_transcript ~= -1));
  idx = find(~isnan(PM_I_n_SQN) & PM_I_transcript ~= -1);
  T1 = mean(abs(PM_I_n_SQN(idx) - PM_I_transcript(idx))) ...
       / mean(abs(PM_I(idx) - PM_I_transcript(idx)))
  T2 = mean((PM_I_n_SQN(idx) - PM_I_transcript(idx)).^2) ...
       / mean((PM_I(idx) - PM_I_transcript(idx)).^2)
  
  genes = intensities_to_genes(PM_I_n_SQN, genes, 'I_n_SQN', OPT.LABEL);
end

if OPT.TEST_SVR,
  % perform SVR based normalization
  [PM_I_n_SVR T1 T2] = init_prediction(PM_I, PM_I_transcript, ...
                                       probe_seq, probe_props, dist, OPT, 'SVR');

  figure
  hold on
  bar([T1' T2'], 'grouped');
  plot([0 21], [1 1], '--k')
  xlabel('intensity quantile');
  ylabel('T_1 / T_2');
  title('I_n-SVR Deviation from constant transcript intensity');
  axis([0 21 0 1.5]);
  print('-depsc', '-r300', '../../PSB_data/norm_probe_results/figs/T1_T2_for_SVR.eps');
  
  fprintf('\nEvaluating transcript normalization with SVR\n');
  fprintf('Evaluating on %2.2f%% of probes:\n', ...
          100*mean(~isnan(PM_I_n_SVR) & PM_I_transcript ~= -1));
  idx = find(~isnan(PM_I_n_SVR) & PM_I_transcript ~= -1);
  T1 = mean(abs(PM_I_n_SVR(idx) - PM_I_transcript(idx))) ...
       / mean(abs(PM_I(idx) - PM_I_transcript(idx)))
  T2 = mean((PM_I_n_SVR(idx) - PM_I_transcript(idx)).^2) ...
       / mean((PM_I(idx) - PM_I_transcript(idx)).^2)
  
  genes = intensities_to_genes(PM_I_n_SVR, genes, 'I_n_SVR', OPT.LABEL);

  clear T1 T2 C idx
end

if OPT.TEST_RR,
  % perform Ridge regression-based normalization
  [PM_I_n_RR T1 T2] = init_prediction(PM_I, PM_I_transcript, ...
                                      probe_seq, probe_props, dist, OPT, 'Ridge');

  figure
  hold on
  bar([T1' T2'], 'grouped');
  plot([0 21], [1 1], '--k')
  xlabel('intensity quantile');
  ylabel('T_1 / T_2');
  title('I_n-RR Deviation from constant transcript intensity');
  axis([0 21 0 1.5]);
  print('-depsc', '-r300', '../../PSB_data/norm_probe_results/figs/T1_T2_for_RR.eps');
  
  fprintf('\nEvaluating transcript normalization with Ridge regression\n');
  fprintf('Evaluating on %2.2f%% of probes:\n', ...
          100*mean(~isnan(PM_I_n_RR) & PM_I_transcript ~= -1));
  idx = find(~isnan(PM_I_n_RR) & PM_I_transcript ~= -1);
  T1 = mean(abs(PM_I_n_RR(idx) - PM_I_transcript(idx))) ...
       / mean(abs(PM_I(idx) - PM_I_transcript(idx)))
  T2 = mean((PM_I_n_RR(idx) - PM_I_transcript(idx)).^2) ...
       / mean((PM_I(idx) - PM_I_transcript(idx)).^2)
  
  genes = intensities_to_genes(PM_I_n_RR, genes, 'I_n_RR', OPT.LABEL);
  clear T1 T2 idx
end


% filter out everything for which there are no test predictions
if ~isempty(OPT.SUBSETS_TO_USE)
  take_idx = find(~isnan(PM_I));
  if OPT.TEST_SQN, 
    take_idx = intersect(take_idx, find(~isnan(PM_I_n_SQN)));
  end
  if OPT.TEST_SVR,
    take_idx = intersect(take_idx, find(~isnan(PM_I_n_SVR)));
  end
  if OPT.TEST_RR,
    take_idx = intersect(take_idx, find(~isnan(PM_I_n_RR)));
  end
  fprintf('%i probes left in test set.\n', length(take_idx));
  
  if OPT.TEST_SQN, 
    PM_I_n_SQN = PM_I_n_SQN(take_idx);
  end
  if OPT.TEST_SVR,
    PM_I_n_SVR = PM_I_n_SVR(take_idx);
  end
  if OPT.TEST_RR,
    PM_I_n_RR = PM_I_n_RR(take_idx);
  end
  
  probe_id = probe_id(take_idx);
  probe_seq = probe_seq(:,take_idx);
  % do not subselect probe_label, because in this case the
  % index from gene struct will fail
  
  PM_I = PM_I(take_idx);
  PM_I_transcript = PM_I_transcript(take_idx);
  if OPT.DIST_CORRECTION,
    dist = dist(take_idx);
  end
  
  gene_idx = get_test_genes(take_idx);
  gene_idx = unique(gene_idx);
  genes = genes(gene_idx);
  fprintf('%i genes left in test set.\n', length(gene_idx));
end


intensities = struct;
intensities.PM_I = PM_I;
OPT.colormap = struct;
OPT.colormap.RAW = [  0   0   0]/256;
if OPT.TEST_SVR
  intensities.PM_I_n_SVR = PM_I_n_SVR;
  OPT.colormap.SVR = [230  30   0]/256;
end
if OPT.TEST_RR
  intensities.PM_I_n_RR = PM_I_n_RR;
  OPT.colormap.RR  = [200 145   0]/256;
end
if OPT.TEST_SQN, 
  intensities.PM_I_n_SQN = PM_I_n_SQN;
  OPT.colormap.SQN = [  0 140  60]/256;
end


if OPT.WRITE_NORMALIZED_I
  fn_probe_ids = ['~/svn/projects/tiling_arrays/segmentation/' ...
                  'segm_probe_info.txt'];
  fn_tmp = 'tmp.txt';
  unix(sprintf('cut -f 2 %s > %s', fn_probe_ids, fn_tmp));
  target_probe_ids = load(fn_tmp);
  unix(sprintf('rm -f %s', fn_tmp));
  
  fn_out = ['~/svn/projects/tiling_arrays/segmentation/' ...
                  'segm_probe_info_IN'];
  array_tag = OPT.fn_arrays{1}(1:8);
  for i=1:length(OPT.fn_arrays),
    assert(equal(array_tag, OPT.fn_arrays{i}(1:8)));
  end
  fn_out = [fn_out '_' array_tag];
  if OPT.USE_TEST_DATA,
    fn_out = [fn_out '_tst'];
  else
    fn_out = [fn_out '_val'];    
  end
  write_normalized_I(intensities, probe_id, fn_out, target_probe_ids)
end

% check to which extent probe sequence effects disappear 
% in normalized intensities
fprintf('Evaluating sequence effects on intensity...\n');
%fhandles = analyze_seq_effects(intensities, probe_seq, probe_label, OPT);

fprintf('Evaluating genes...\n');
[eval T1 T2 au_ROC] = eval_genes(genes, probe_label, OPT.LABEL, ...
                                 OPT.NUM_QUANTILES);
% now that genes are no longer needed subselect probe_label, too
clear genes
if ~isempty(OPT.SUBSETS_TO_USE)
  probe_label = probe_label(take_idx);
end

ige_idx = find(probe_label==OPT.LABEL.intergenic);
if isfield(eval, 'SVR'),
  T1.SVR(end+1) = mean(abs(PM_I_n_SVR(ige_idx) - median(PM_I_n_SVR(ige_idx)))) ...
        / median(PM_I(ige_idx));
  T2.SVR(end+1) = sqrt(mean(abs(PM_I_n_SVR(ige_idx) - mean(PM_I_n_SVR(ige_idx))))) ...
        / mean(PM_I(ige_idx));

  figure
  bar([T1.SVR' T2.SVR'], 'grouped')
  title('SVR');
  xlabel('expression quantile');
  ylabel('Error relative to transcript intensity');
  xtl = get(gca, 'XTickLabel');
  xtl(end-1,1:3) = 'INO';
  xtl(end,1:3) = 'IGE';
  set(gca, 'XTickLabel', xtl);
  axis([0 OPT.NUM_QUANTILES+3 0 0.25]);
end
if isfield(eval, 'RR'),
  T1.RR(end+1) = mean(abs(PM_I_n_RR(ige_idx) - median(PM_I_n_RR(ige_idx)))) ...
        / median(PM_I(ige_idx));
  T2.RR(end+1) = sqrt(mean(abs(PM_I_n_RR(ige_idx) - mean(PM_I_n_RR(ige_idx))))) ...
        / mean(PM_I(ige_idx));

  figure
  bar([T1.RR' T2.RR'], 'grouped')
  title('RR');
  xlabel('expression quantile');
  ylabel('Error relative to transcript intensity');
  xtl = get(gca, 'XTickLabel');
  xtl(end-1,1:3) = 'INO';
  xtl(end,1:3) = 'IGE';
  set(gca, 'XTickLabel', xtl);
  axis([0 OPT.NUM_QUANTILES+3 0 0.25]);
end
if isfield(eval, 'SQN'),
  T1.SQN(end+1) = mean(abs(PM_I_n_SQN(ige_idx) - median(PM_I_n_SQN(ige_idx)))) ...
        / median(PM_I(ige_idx));
  T2.SQN(end+1) = sqrt(mean(abs(PM_I_n_SQN(ige_idx) - mean(PM_I_n_SQN(ige_idx))))) ...
        / mean(PM_I(ige_idx));

  figure
  bar([T1.SQN' T2.SQN'], 'grouped')
  title('SQN');
  xlabel('expression quantile');
  ylabel('Error relative to transcript intensity');
  xtl = get(gca, 'XTickLabel');
  xtl(end-1,1:3) = 'INO';
  xtl(end,1:3) = 'IGE';
  set(gca, 'XTickLabel', xtl);
  axis([0 OPT.NUM_QUANTILES+3 0 0.25]);
end


figure
bar([au_ROC.RAW' au_ROC.SQN' au_ROC.RR' au_ROC.SVR'], 'grouped') 
axis([0 OPT.NUM_QUANTILES+1 0 1]);
title('au ROC averaged over genes');
xlabel('Expression quantile');
ylabel('Mean area under the ROC curve');
print('-depsc', '-r300', '../../PSB_data/norm_probe_results/figs/auROC_gene_mean.eps');

% check separability between exonic and intronic/intergenic probes
% using naive per-probe thresholding
fhandles = analyze_separability(intensities, probe_seq, probe_label, OPT);








